/*
  File Name     : ft602_hdmi_rx.v 
  Department    : IC Design, FTDI SGP
  Author        : Do Ngoc Duong
  History       : 16 August 2016 

  Description   : This module receives video from HDMI card or 
                  from pattern generator to double pixel bus 
*/
module ft602_hdmi_rx (  
  //IO interface 
  input  rst_n,
  input  gen_ena,
  output frame_drop, 
  // HDMI card interface  
  input  hdmi_pclk,
  input  hdmi_vsync,
  input  hdmi_hsync,
  input  hdmi_de,
  input  [11:0] hdmi_rd,
  input  [11:0] hdmi_gd,
  input  [11:0] hdmi_bd,
  // Pattern generator interface  
  input seq_ena,
  input [31:0] seq_data,
  //  
  input  gen_pclk,
  input  gen_vsync,
  input  gen_hsync,
  input  gen_de,
  input  [35:0] gen_data, 
  //write FIFO interface 
  output out_clk,
  input  wrmark,
  output fifowr,
  output [35:0] wdata
);
//****************
localparam MAXCLK = 22'd2475000; //0x25C3F8  
//****************
//Latch HDMI input 
reg hdmi_vsyncp1;
reg hdmi_hsyncp1;
reg hdmi_dep1;
reg [11:0] hdmi_rdp1;
reg [11:0] hdmi_gdp1;
reg [11:0] hdmi_bdp1;
//
always @ (posedge hdmi_pclk or negedge rst_n)
begin
  if (!rst_n)
  begin 
    hdmi_vsyncp1 <= 1'b1;
    hdmi_hsyncp1 <= 1'b1;
    hdmi_dep1    <= 1'b0;
    hdmi_rdp1    <= 12'h000;
    hdmi_gdp1    <= 12'h000;
    hdmi_bdp1    <= 12'h000;
  end
  else 
  begin
    hdmi_vsyncp1 <= hdmi_vsync;
    hdmi_hsyncp1 <= hdmi_hsync;
    hdmi_dep1    <= hdmi_de;   
    hdmi_rdp1    <= hdmi_rd;  
    hdmi_gdp1    <= hdmi_gd;  
    hdmi_bdp1    <= hdmi_bd;  
  end
end
//Select inputs between internal pattern generator or external HDMI card
wire clk;
wire vsync;
wire hsync;
wire de;
wire [35:0] idata; 
//
assign clk     = gen_ena ? gen_pclk    : hdmi_pclk;
assign vsync   = gen_ena ? gen_vsync   : hdmi_vsyncp1;
assign hsync   = gen_ena ? gen_hsync   : hdmi_hsyncp1;
assign de      = gen_ena ? gen_de      : hdmi_dep1;
assign idata   = gen_ena ? gen_data    : {hdmi_rdp1,hdmi_gdp1,hdmi_bdp1};
//
assign out_clk = clk; 
//Select parameters for video formats
// 
reg video_vld;  
reg video_vld_p1;  
reg vsync_p1;
reg hsync_p1;
reg de_p1;
reg [35:0] idata_p1;
reg [31:0] seq_data_p1;
reg vsync_p2;
reg hsync_p2;
//delay 
wire frame_start;
wire frame_end;
wire line_start;
wire time_out; 
wire video_vld_ne; 
//
reg  frame_startp1;
reg  frame_startp2;
//
always @ (posedge clk or negedge rst_n)
begin
  if(!rst_n)
  begin 
    vsync_p1 <= 1'b1;
    hsync_p1 <= 1'b1; 
    de_p1    <= 1'b0;
    idata_p1 <= 36'd0;
	 seq_data_p1 <= 32'd0;
    //
    vsync_p2 <= 1'b1;
    hsync_p2 <= 1'b1;
    video_vld_p1 <= 1'b0;
    frame_startp1<= 1'b0; 
    frame_startp2<= 1'b0; 
  end 
  else 
  begin 
    vsync_p1 <= vsync;
    hsync_p1 <= hsync;
    de_p1    <= de;
    idata_p1 <= idata;
	 seq_data_p1 <= seq_data;
    // 
    vsync_p2 <= vsync_p1;
    hsync_p2 <= hsync_p1;
    video_vld_p1 <= video_vld;
    frame_startp1<= frame_start;  
    frame_startp2<= frame_startp1;  
  end
end
//
reg [10:0] pixl_cnt;
reg [21:0] tout_cnt; 
assign frame_start = vsync_p2 & (!vsync_p1);
assign frame_end   = vsync_p1 & (!vsync_p2); 
assign line_start  = hsync_p2 & (!hsync_p1);
assign time_out    = (tout_cnt == MAXCLK);
assign video_vld_ne= video_vld_p1 & (!video_vld);  
//Pixel counter  
always @ (posedge clk or negedge rst_n)
begin
  if (!rst_n)
    pixl_cnt <= 11'd0;
  else if (frame_start | line_start | time_out | frame_end)
    pixl_cnt <= 11'd0;
  else if (de_p1 & video_vld)
    pixl_cnt <= pixl_cnt + 1'b1;
end 
//Time out couter 
always @ (posedge clk or negedge rst_n)
begin
  if (!rst_n)
    tout_cnt <= 22'd0;
  else if (frame_start | frame_end | time_out) 
    tout_cnt <= 22'd0;
  else if (video_vld) 
    tout_cnt <= tout_cnt + 1'b1;
end
//
wire bit_flip;
assign bit_flip = pixl_cnt[0];
//
reg [35:0] comb_data;
reg        comb_done; 
//
always @ (posedge clk or negedge rst_n)
begin
  if (!rst_n)
  begin 
    comb_data <= 36'd0;
    comb_done <= 1'b0;
  end 
  else if (video_vld_ne)
  begin
    comb_data <= {4'h1,32'h0000_0055};
    comb_done <= 1'b1;
  end   
  else if (frame_start & (!wrmark))
  begin 
    comb_data <= {4'hF,32'h0000_820C};
    comb_done <= 1'b1; 
  end 
  else if ((frame_startp1 | frame_startp2) & (!wrmark))
  begin 
    comb_data <= {4'hF,32'hBABE_FACE};
    comb_done <= 1'b1; 
  end 
  else if (de_p1 & video_vld & (!wrmark))
    if (seq_ena)
	 begin
      comb_data <= {4'hF,seq_data_p1};
      comb_done <= bit_flip; //1'b1;
	 end
    else 
	 begin
	   if (bit_flip)
      begin 
        comb_data <= {4'hF,idata_p1[35:28],idata_p1[23:16],comb_data[15:0]};	 
        comb_done <= 1'b1;
      end 
      else 
      begin      
        comb_data <= {4'hF,16'h0000,idata_p1[35:28],idata_p1[23:16]};
        comb_done <= 1'b0;
      end 
    end		
  else 
  begin 
    comb_data <= 36'd0;
    comb_done <= 1'b0;
  end 
end
// 
assign fifowr = comb_done;
assign wdata  = comb_data; 
//
reg  stop_writing; 
//
always @ (posedge clk or negedge rst_n)
begin
  if (!rst_n)
    stop_writing <= 1'b0;
  else if (wrmark & de)
    stop_writing <= 1'b1;
  else if ((!wrmark) & frame_end)
    stop_writing <= 1'b0;
end 
// 
//Video window
always @ (posedge clk or negedge rst_n)
begin
  if (!rst_n)
    video_vld <= 1'b0;
  else if (frame_end | time_out | (stop_writing & de_p1))
    video_vld <= 1'b0; 
  else if (frame_start & (!wrmark)) //(!stop_writing))
    video_vld <= 1'b1;
end 
//
reg iframe_drop;
always @ (posedge clk or negedge rst_n)
begin
  if (!rst_n)
    iframe_drop <= 1'b0;
  else if (de_p1 & stop_writing)
    iframe_drop <= 1'b1;
  else if (frame_end)
    iframe_drop <= 1'b0;
end
// 
assign frame_drop = iframe_drop; 
//    
endmodule 
